package com.savagelook 
{
	import flash.utils.ByteArray;
	/**
	 * ...
	 * @author Tony Lukasavage - SavageLook.com
	 */
	public class SwatchLoader
	{
		private const FILE_SIGNATURE:String = "ASEF";
		private const ENCODING:String = "iso-8859-1";
		
		private var _version_major:uint;
		private var _version_minor:uint;
		private var _number_of_blocks:uint;
		public var _groups:Vector.<SwatchGroup> = new Vector.<SwatchGroup>();

		public function SwatchLoader(ase:ByteArray = null) {
			if (ase) {
				loadSwatch(ase);
			} else {
				_reset();
			}
		}
		
		public function loadSwatch(ase:ByteArray):Boolean {
			var blockLength:uint, nameLength:uint, i:uint;
			var group:SwatchGroup;
			_reset();
			
			if (ase.length < 12) {
				//throw new Error("file size too small");
				return false;
			}
			
			var signature:String = ase.readMultiByte(4, ENCODING);
			if (signature != FILE_SIGNATURE) {
				//throw new Error("invalid file signature");
				return false;
			}
			
			_version_major = ase.readShort();
			_version_minor = ase.readShort();
			_number_of_blocks = ase.readInt();
			
			
			while (ase.bytesAvailable > 0) {
				var chunk:uint = ase.readUnsignedShort();
				if (0xC001 == chunk) {
					group = new SwatchGroup();
					_groups.push(group);
					blockLength = ase.readUnsignedInt();
					nameLength = ase.readUnsignedShort();
					
					for (i = 0; i < nameLength; i++) {
						group.name += String.fromCharCode(ase.readUnsignedShort());
					}
				} else if (0x0001 == chunk) {
					if (_groups.length <= 0) {
						_groups.push(new SwatchGroup());
					} 
					group = _groups[_groups.length-1];
					
					blockLength = ase.readUnsignedInt();
					nameLength = ase.readUnsignedShort();
					
					var color:String = "";
					for (i = 0; i < nameLength; i++) {
						color += String.fromCharCode(ase.readUnsignedShort());
					}
					group.colorNames.push(color);
					
					var colorMode:String = _trim(ase.readMultiByte(4, ENCODING));
					var red:uint, green:uint, blue:uint, type:uint;
					if (colorMode == "RGB") {
						red = Math.round(ase.readFloat() * 255.0);
						green  = Math.round(ase.readFloat() * 255.0);
						blue  = Math.round(ase.readFloat() * 255.0);
						type = ase.readUnsignedShort();
					} else if (colorMode == "CMYK") {
						var c:Number = ase.readFloat();
						var m:Number  = ase.readFloat();
						var y:Number  = ase.readFloat();
						var k:Number = ase.readFloat();
						red =   Math.round((1.0 - c) * (1.0 - k) * 255.0);
						green = Math.round((1.0 - m) * (1.0 - k) * 255.0);
						blue =  Math.round((1.0 - y) * (1.0 - k) * 255.0);
						type = ase.readUnsignedShort();
					} else if (colorMode == "Gray") {
						var gray:Number = ase.readFloat();
						red = 255.0 * gray;
						green = 255.0 * gray;
						blue = 255.0 * gray;
						type = ase.readUnsignedShort();
					} else {
						//throw new Error("unhandled color mode");
						return false;
					}
					group.colorTypes.push(type);
					group.colorValues.push((red * 256 * 256) + (green * 256) + blue);
				} else {
					//throw new Error("invalid ASE format");
					return false;
				}
			}
			if (_groups.length <= 0) {
				return false;
			}
			return true;
		}
		
		public function get name():String {
			if (_groups.length > 0) {
				return _groups[0].name;
			} else {
				return "";
			}
		}
		
		public function get colorNames():Vector.<String> {
			if (_groups.length > 0) {
				return _groups[0].colorNames;
			} else {
				return new Vector.<String>();
			}
		}
		
		public function get colorValues():Vector.<uint> {
			if (_groups.length > 0) {
				return _groups[0].colorValues;
			} else {
				return new Vector.<uint>();
			}
		}
		
		public function get groups():Vector.<SwatchGroup> {
			return _groups;
		}
		
		private function _trim( s:String ):String {
			s = s.replace(/^\s*/, "");
			return s.replace(/\s*$/, "");
		}
		
		private function _reset():void 
		{
			_version_major = 0;
			_version_minor = 0;
			_number_of_blocks = 0;
			_groups.length = 0;
		}
	}
}

class SwatchGroup {
	public var name:String = "";
	public var colorNames:Vector.<String> = new Vector.<String>();
	public var colorValues:Vector.<uint> = new Vector.<uint>();
	public var colorTypes:Vector.<uint> = new Vector.<uint>();
}